// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:cli_completion/cli_completion.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:wyatt_continuous_deployment/src/core/commands/commands.dart';
import 'package:wyatt_continuous_deployment/src/core/constants/app_constants.dart';
import 'package:wyatt_continuous_deployment/src/core/enums/flag.dart';
import 'package:wyatt_continuous_deployment/src/core/utils/logging.dart';
import 'package:wyatt_continuous_deployment/src/version.dart';

class WyattContinuousDeploymentCommandRunner
    extends CompletionCommandRunner<int> {
  WyattContinuousDeploymentCommandRunner()
      : super(
          AppConstants.executableName,
          '🍺 Where all your dreams come to life',
        ) {
    argParser.addFlags();
    addCommand(InitProjectCommand());
    addCommand(BuildAppCommand());
    addCommand(DeployAppCommand());
    addCommand(BuildAndDeployAppCommand());
  }

  @override
  Future<int> run(Iterable<String> args) async {
    try {
      final results = parse(args);

      logger.level = results['verbose'] == true ? Level.verbose : Level.info;

      return await runCommand(results) ?? ExitCode.success.code;
    } on FormatException catch (e) {
      logger
        ..err(e.message)
        ..info('')
        ..info(usage);
      return ExitCode.usage.code;
    } on UsageException catch (e) {
      logger
        ..err(e.message)
        ..info('')
        ..info(e.usage);
      return ExitCode.usage.code;
    } on ProcessException catch (error) {
      logger.err(error.message);
      return ExitCode.unavailable.code;
    } catch (error) {
      logger.err('$error');
      return ExitCode.software.code;
    }
  }

  @override
  Future<int?> runCommand(ArgResults topLevelResults) async {
    if (topLevelResults.command?.name == 'completion') {
      await super.runCommand(topLevelResults);
      return ExitCode.success.code;
    }

    int? exitCode = ExitCode.unavailable.code;
    if (topLevelResults['version'] == true) {
      logger.info(packageVersion);
      exitCode = ExitCode.success.code;
    } else {
      exitCode = await super.runCommand(topLevelResults);
    }
    return exitCode;
  }
}

extension on ArgParser {
  void addFlags() {
    addFlag(
      AppConstants.versionArg,
      negatable: false,
      help: 'Print the current version.',
    );
    addFlag(
      Flag.verbose.name,
      abbr: Flag.verbose.abbr,
      negatable: false,
      help: 'Print verbose output.',
    );
    addOption(
      AppConstants.configArg,
      abbr: 'c',
      help: 'Path to the wyatt.yaml file.',
      valueHelp: 'path',
    );
  }
}
